function avarb = gmmvarHAC(y,z,x,heter,hac_lag);
%avarb = gmmvarHAC(y,z,x,heter,hac_lag);


%Calculates variance of the coefficient vector for GMM with optimal weighting matrix (See Hayashi, chp.3)
%y is (T*1) dependent variable
%z is (T*l) independent variable
%x is (T*k) matrix of instruments
%hac_lag is the n. of Newey West bandwidth
%heter is a dummy: =0 if want cond. homosked. and =1 if cond. heterosked.
%we checked that if x=z then we get back to the OLS if cond homosk

%check order condition
T=length(y); Sxx=x'*x/T; Sxz=x'*z/T; Sxy=x'*y/T; [a,k]=size(x); [a,l]=size(z);
if k<l; disp('order condition violated'); 
    else;
    %1st stage (2SLS)
    W1=inv(Sxx);
    b1=(inv(Sxz'*W1*Sxz))*(Sxz'*W1*Sxy);
    e=y-z*b1;
    if heter==1; 
        g_t=mmult(e,x);     
        S=nw(g_t,hac_lag); 
        W2=inv(S); 
    elseif heter==0; 
        S=(Sxx)*sum(e.^2)/T; W2=inv(S); %%%%%%
    end;
    %2nd stage
    b2=(inv(Sxz'*W2*Sxz))*(Sxz'*W2*Sxy);
    if heter==1; 
        avarb=inv(Sxz'*W2*Sxz); 
    elseif heter==0; 
        avarb=(inv((Sxz)'*inv(Sxx)*(Sxz)))*sum((y-z*b2).^2)/T; %%%%%
    end;
end;

%-----------------------
function result=mmult(x,y);

%Multiplies a vector x by each of the columns of y
%INPUT:  x, an nx1 vector
%        y, an nxk matrix

result=[];
for k=1:size(y,2);
    result=[result,x.*y(:,k)];
end;
    

function result = nw(y,qn);
%input: y is a T*k vector and qn is the truncation lag
%output: the newey west HAC covariance estimator 
%Formulas are from Hayashi

[T,k]=size(y); ybar=ones(T,1)*((sum(y))/T);
dy=y-ybar;
G0=dy'*dy/T;
for j=1:qn-1;
   gamma=(dy(j+1:T,:)'*dy(1:T-j,:))./(T-1);
   G0=G0+(gamma+gamma').*(1-abs(j/qn));
end;
result=G0;