function [irf_inflation,irf_output,IRF_inflation,IRF_output]=var(y,p,H,M)

[T0,n] = size(y); 
J = [eye(n) zeros(n,n*(p-1))];
T = T0-p;

% GLS estimation of VARX model
X=ones(T,1);	
for i=1:p
 	X=[X y(p-i+1:T0-i,:)];
end
Bhat = inv(X'*X)*X'*y(p+1:T0,:);
uhat = y(p+1:T0,:)-X*Bhat;
Sigmahat = uhat'*uhat/(T-1+p*n*n);
P = chol(Sigmahat)';

% Impulse responses to exogenous changes in the yield curve
Phi = [Bhat(2:end,:)';eye(n*(p-1)) zeros(n*(p-1),n)];
irf_inflation = zeros(1,n-2);
irf_output    = zeros(1,n-2);
for i=1:H
    F = J*Phi^i*J';
    irf = F(1:n,1:n)*P;
    irf_inflation = [irf_inflation;irf(1,3:n)];
    irf_output    = [irf_output;irf(2,3:n)];
end

IRF_inflation=[];
IRF_output=[];
ystar = zeros(T0,n);
for i=1:M
    % Bootstrapping the GLS estimator
    indx  = floor(T*rand(T,1))+1;
    ustar = uhat(indx,:);
    ystar(1:p,1:n) = y(1:p,1:n);
    for t=p+1:T0
        x = 1;
        for j=1:p
 	    x = [x ystar(t-j,:)];
        end
        ystar(t,:) = x*Bhat+ustar(t-p,:);
    end
    X=ones(T,1);	
    for j=1:p
 	X=[X ystar(p-j+1:T0-j,:)];
    end
    Bstar = inv(X'*X)*X'*ystar(p+1:T0,:);
    ustar = ystar(p+1:T0,:)-X*Bstar;
    Sigmastar = ustar'*ustar/(T-1+p*n*n);
    P = chol(Sigmastar)';

    % Bootstrapping impulse responses
    Phi = [Bstar(2:end,:)';eye(n*(p-1)) zeros(n*(p-1),n)];
    irf1 = zeros(1,n-2);
    irf2 = zeros(1,n-2);
    for j=1:H
         F = J*Phi^j*J';
         irf = F(1:n,1:n)*P;
         irf1 = [irf1;irf(1,3:n)];
         irf2 = [irf2;irf(2,3:n)];
    end
    IRF_inflation = [IRF_inflation;reshape(irf1,1,(n-2)*(H+1))];
    IRF_output = [IRF_output;reshape(irf2,1,(n-2)*(H+1))];
end

save uhat uhat -ascii; 